@@ -5,20 +5,19 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+from __future__ import annotations
 
 import datetime
 import errno
 import json
 import traceback
 
-from mercurial.hgweb import (
-    common as hgwebcommon,
-)
+from mercurial.hgweb import common as hgwebcommon
 
 from mercurial import (
+    exthelper,
     pycompat,
-    util,
+    wireprotoserver,
 )
 
 from . import blobstore
@@ -31,6 +30,10 @@ HTTP_METHOD_NOT_ALLOWED = hgwebcommon.HTTP_METHOD_NOT_ALLOWED
 HTTP_NOT_ACCEPTABLE = hgwebcommon.HTTP_NOT_ACCEPTABLE
 HTTP_UNSUPPORTED_MEDIA_TYPE = hgwebcommon.HTTP_UNSUPPORTED_MEDIA_TYPE
 
+eh = exthelper.exthelper()
+
+
+@eh.wrapfunction(wireprotoserver, 'handlewsgirequest')
 def handlewsgirequest(orig, rctx, req, res, checkperm):
     """Wrap wireprotoserver.handlewsgirequest() to possibly process an LFS
     request if it is left unprocessed by the wrapped method.
@@ -38,10 +41,10 @@ def handlewsgirequest(orig, rctx, req, res, checkperm):
     if orig(rctx, req, res, checkperm):
         return True
 
-    if not rctx.repo.ui.configbool('experimental', 'lfs.serve'):
+    if not rctx.repo.ui.configbool(b'experimental', b'lfs.serve'):
         return False
 
-    if not util.safehasattr(rctx.repo.svfs, 'lfslocalblobstore'):
+    if not hasattr(rctx.repo.svfs, 'lfslocalblobstore'):
         return False
 
     if not req.dispatchpath:
@@ -49,14 +52,14 @@ def handlewsgirequest(orig, rctx, req, res, checkperm):
 
     try:
         if req.dispatchpath == b'.git/info/lfs/objects/batch':
-            checkperm(rctx, req, 'pull')
+            checkperm(rctx, req, b'pull')
             return _processbatchrequest(rctx.repo, req, res)
         # TODO: reserve and use a path in the proposed http wireprotocol /api/
         #       namespace?
         elif req.dispatchpath.startswith(b'.hg/lfs/objects'):
-            return _processbasictransfer(rctx.repo, req, res,
-                                         lambda perm:
-                                                checkperm(rctx, req, perm))
+            return _processbasictransfer(
+                rctx.repo, req, res, lambda perm: checkperm(rctx, req, perm)
+            )
         return False
     except hgwebcommon.ErrorResponse as e:
         # XXX: copied from the handler surrounding wireprotoserver._callhttp()
@@ -68,23 +71,27 @@ def handlewsgirequest(orig, rctx, req, res, checkperm):
         res.setbodybytes(b'0\n%s\n' % pycompat.bytestr(e))
         return True
 
+
 def _sethttperror(res, code, message=None):
     res.status = hgwebcommon.statusmessage(code, message=message)
     res.headers[b'Content-Type'] = b'text/plain; charset=utf-8'
     res.setbodybytes(b'')
 
+
 def _logexception(req):
     """Write information about the current exception to wsgi.errors."""
     tb = pycompat.sysbytes(traceback.format_exc())
-    errorlog = req.rawenv[r'wsgi.errors']
+    errorlog = req.rawenv[b'wsgi.errors']
 
     uri = b''
     if req.apppath:
         uri += req.apppath
     uri += b'/' + req.dispatchpath
 
-    errorlog.write(b"Exception happened while processing request '%s':\n%s" %
-                   (uri, tb))
+    errorlog.write(
+        b"Exception happened while processing request '%s':\n%s" % (uri, tb)
+    )
+
 
 def _processbatchrequest(repo, req, res):
     """Handle a request for the Batch API, which is the gateway to granting file
@@ -125,24 +132,36 @@ def _processbatchrequest(repo, req, res):
         return True
 
     # XXX: specify an encoding?
-    lfsreq = json.loads(req.bodyfh.read())
+    lfsreq = pycompat.json_loads(req.bodyfh.read())
 
     # If no transfer handlers are explicitly requested, 'basic' is assumed.
     if 'basic' not in lfsreq.get('transfers', ['basic']):
-        _sethttperror(res, HTTP_BAD_REQUEST,
-                      b'Only the basic LFS transfer handler is supported')
+        _sethttperror(
+            res,
+            HTTP_BAD_REQUEST,
+            b'Only the basic LFS transfer handler is supported',
+        )
         return True
 
     operation = lfsreq.get('operation')
-    if operation not in ('upload', 'download'):
-        _sethttperror(res, HTTP_BAD_REQUEST,
-                      b'Unsupported LFS transfer operation: %s' % operation)
+    operation = pycompat.bytestr(operation)
+
+    if operation not in (b'upload', b'download'):
+        _sethttperror(
+            res,
+            HTTP_BAD_REQUEST,
+            b'Unsupported LFS transfer operation: %s' % operation,
+        )
         return True
 
     localstore = repo.svfs.lfslocalblobstore
 
-    objects = [p for p in _batchresponseobjects(req, lfsreq.get('objects', []),
-                                                operation, localstore)]
+    objects = [
+        p
+        for p in _batchresponseobjects(
+            req, lfsreq.get('objects', []), operation, localstore
+        )
+    ]
 
     rsp = {
         'transfer': 'basic',
@@ -155,6 +174,7 @@ def _processbatchrequest(repo, req, res):
 
     return True
 
+
 def _batchresponseobjects(req, objects, action, store):
     """Yield one dictionary of attributes for the Batch API response for each
     object in the list.
@@ -185,11 +205,12 @@ def _batchresponseobjects(req, objects, action, store):
 
     for obj in objects:
         # Convert unicode to ASCII to create a filesystem path
-        oid = obj.get('oid').encode('ascii')
+        soid = obj.get('oid')
+        oid = soid.encode('ascii')
         rsp = {
-            'oid': oid,
+            'oid': soid,
             'size': obj.get('size'),  # XXX: should this check the local size?
-            #'authenticated': True,
+            # 'authenticated': True,
         }
 
         exists = True
@@ -204,7 +225,7 @@ def _batchresponseobjects(req, objects, action, store):
         # verified as the file is streamed to the caller.
         try:
             verifies = store.verify(oid)
-            if verifies and action == 'upload':
+            if verifies and action == b'upload':
                 # The client will skip this upload, but make sure it remains
                 # available locally.
                 store.linkfromusercache(oid)
@@ -214,7 +235,7 @@ def _batchresponseobjects(req, objects, action, store):
 
                 rsp['error'] = {
                     'code': 500,
-                    'message': inst.strerror or 'Internal Server Server'
+                    'message': inst.strerror or 'Internal Server Server',
                 }
                 yield rsp
                 continue
@@ -223,19 +244,19 @@ def _batchresponseobjects(req, objects, action, store):
 
         # Items are always listed for downloads.  They are dropped for uploads
         # IFF they already exist locally.
-        if action == 'download':
+        if action == b'download':
             if not exists:
                 rsp['error'] = {
                     'code': 404,
-                    'message': "The object does not exist"
+                    'message': "The object does not exist",
                 }
                 yield rsp
                 continue
 
             elif not verifies:
                 rsp['error'] = {
-                    'code': 422,   # XXX: is this the right code?
-                    'message': "The object is corrupt"
+                    'code': 422,  # XXX: is this the right code?
+                    'message': "The object is corrupt",
                 }
                 yield rsp
                 continue
@@ -250,20 +271,20 @@ def _batchresponseobjects(req, objects, action, store):
             # The spec doesn't mention the Accept header here, but avoid
             # a gratuitous deviation from lfs-test-server in the test
             # output.
-            hdr = {
-                'Accept': 'application/vnd.git-lfs'
-            }
+            hdr = {'Accept': 'application/vnd.git-lfs'}
 
-            auth = req.headers.get('Authorization', '')
-            if auth.startswith('Basic '):
-                hdr['Authorization'] = auth
+            auth = req.headers.get(b'Authorization', b'')
+            if auth.startswith(b'Basic '):
+                hdr['Authorization'] = pycompat.strurl(auth)
 
             return hdr
 
         rsp['actions'] = {
-            '%s' % action: {
-                'href': '%s%s/.hg/lfs/objects/%s'
-                    % (req.baseurl, req.apppath, oid),
+            '%s'
+            % pycompat.strurl(action): {
+                'href': pycompat.strurl(
+                    b'%s%s/.hg/lfs/objects/%s' % (req.baseurl, req.apppath, oid)
+                ),
                 # datetime.isoformat() doesn't include the 'Z' suffix
                 "expires_at": expiresat.strftime('%Y-%m-%dT%H:%M:%SZ'),
                 'header': _buildheader(),
@@ -272,6 +293,7 @@ def _batchresponseobjects(req, objects, action, store):
 
         yield rsp
 
+
 def _processbasictransfer(repo, req, res, checkperm):
     """Handle a single file upload (PUT) or download (GET) action for the Basic
     Transfer Adapter.
@@ -292,7 +314,7 @@ def _processbasictransfer(repo, req, res, checkperm):
         return True
 
     if method == b'PUT':
-        checkperm('upload')
+        checkperm(b'upload')
 
         # TODO: verify Content-Type?
 
@@ -304,7 +326,7 @@ def _processbasictransfer(repo, req, res, checkperm):
 
         statusmessage = hgwebcommon.statusmessage
         try:
-            localstore.download(oid, req.bodyfh)
+            localstore.download(oid, req.bodyfh, req.headers[b'Content-Length'])
             res.status = statusmessage(HTTP_OK if existed else HTTP_CREATED)
         except blobstore.LfsCorruptionError:
             _logexception(req)
@@ -319,7 +341,7 @@ def _processbasictransfer(repo, req, res, checkperm):
 
         return True
     elif method == b'GET':
-        checkperm('pull')
+        checkperm(b'pull')
 
         res.status = hgwebcommon.statusmessage(HTTP_OK)
         res.headers[b'Content-Type'] = b'application/octet-stream'
@@ -339,6 +361,9 @@ def _processbasictransfer(repo, req, res, checkperm):
 
         return True
     else:
-        _sethttperror(res, HTTP_METHOD_NOT_ALLOWED,
-                      message=b'Unsupported LFS transfer method: %s' % method)
+        _sethttperror(
+            res,
+            HTTP_METHOD_NOT_ALLOWED,
+            message=b'Unsupported LFS transfer method: %s' % method,
+        )
         return True
